package com.inneractive.api.ads.sample;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.Toast;

import com.fyber.inneractive.sdk.external.ImpressionData;
import com.fyber.inneractive.sdk.external.OnGlobalImpressionDataListener;
import com.fyber.inneractive.sdk.external.InneractiveAdManager;
import com.fyber.inneractive.sdk.external.InneractiveAdRequest;
import com.fyber.inneractive.sdk.external.InneractiveAdSpot;
import com.fyber.inneractive.sdk.external.InneractiveAdSpotManager;
import com.fyber.inneractive.sdk.external.InneractiveAdViewEventsListener;
import com.fyber.inneractive.sdk.external.InneractiveAdViewEventsListenerWithImpressionData;
import com.fyber.inneractive.sdk.external.InneractiveAdViewUnitController;
import com.fyber.inneractive.sdk.external.InneractiveAdViewVideoContentController;
import com.fyber.inneractive.sdk.external.InneractiveErrorCode;
import com.fyber.inneractive.sdk.external.InneractiveFullscreenAdEventsListenerWithImpressionData;
import com.fyber.inneractive.sdk.external.InneractiveFullscreenUnitController;
import com.fyber.inneractive.sdk.external.InneractiveFullscreenVideoContentController;
import com.fyber.inneractive.sdk.external.InneractiveUnitController.AdDisplayError;
import com.fyber.inneractive.sdk.external.InneractiveUserConfig;
import com.fyber.inneractive.sdk.external.VideoContentListener;


/**
 * Main sample application activity
 */
public class IaSampleAdActivity extends Activity {
	// Definitions
	protected final static String INNERACTIVE_LOG_TAG = "InneractiveSampleApp";
	static String SAMPLE_APP_ID = "102960";

	static String SAMPLE_BANNER_SPOT_ID = "150942";
	static String SAMPLE_RECTANGLE_SPOT_ID = "150943";
	static String SAMPLE_INTERSTITIAL_SPOT_ID = "150946";

	// Members
	// First tab UI elements - Dynamically created views

	/** The banner spot */
	InneractiveAdSpot mBannerSpot;
	/** The interstitial spot */
	InneractiveAdSpot mInterstitialSpot;

	/** Banner's app id input field */
	EditText bannerAppIdText;
	/** Interstitial's app id input field */
	EditText interstitialAppIdText;
	/** Banner's spot id input field */
	EditText bannerSpotIdText;
	/** Interstitial's spot id input field */
	EditText interstitialSpotIdText;

	/** For showing sample events */
	private Toast mToast;

	/** The current app id in use */
	private String mCurrentAppId;


	/** Activity's entry point. Good place to initialize the Inneractive library */
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
    	setContentView(R.layout.tab_layout);

		// Update the window title with the SDK version
		this.setTitle("Inneractive Sample App. SDK v= " + InneractiveAdManager.getVersion());
    	    	
    	initializeUI();
    	configureGlobalImpressionDataListener();
    }

	private void configureGlobalImpressionDataListener() {
		InneractiveAdManager.setImpressionDataListener(new OnGlobalImpressionDataListener() {
			@Override
			public void onImpression(String spotId, String unitId, ImpressionData impressionData) {
				Log.d(INNERACTIVE_LOG_TAG, "spotId : " + spotId + ",unitId: " + unitId + ", impressionData: " + impressionData);
			}
		});
	}


	/** populate tab control */
    void initializeUI() {
    	TabHost tabs=(TabHost)findViewById(R.id.tabhost);
    	tabs.setup();

		// First tab is used for dynamically created banners
    	TabHost.TabSpec spec=tabs.newTabSpec("tag1"); 
    	spec.setContent(R.id.tab1);
       	spec.setIndicator("Banner");
    	tabs.addTab(spec);

		// Third tab is used for interstitial ads
    	spec=tabs.newTabSpec("Interstitial"); 
    	spec.setContent(R.id.tab2);
    	spec.setIndicator("Interstitial");
    	tabs.addTab(spec);
    	tabs.setCurrentTab(0);

		// Used for caching
		bannerAppIdText = (EditText) findViewById(R.id.bannerAppId);
    	bannerSpotIdText = (EditText) findViewById(R.id.bannerSpotId);
		interstitialAppIdText = (EditText) findViewById(R.id.interstitialAppId);
    	interstitialSpotIdText = (EditText)findViewById(R.id.interstitialSpotId);

		// Update the window title with the SDK version
		updateTitle("Inneractive Sample App. SDK v= " + InneractiveAdManager.getVersion());
    }

	/**
	 * Helper method for initializing the SDK with the given app id
	 * @param appId The app id provided through the Inneractive console
	 */
	protected void initInneractiveSdk(String appId) {

		// Get app id from input field
		if (TextUtils.isEmpty(appId)) {
			appId = SAMPLE_APP_ID;
		}

		boolean appIdHasChanged = !TextUtils.equals(mCurrentAppId, appId);
		if ( !InneractiveAdManager.wasInitialized() || appIdHasChanged) {

			//The following is to ease testing purposes. there is no need to destroy the SDK for every request.
			//Please refer to the documentation regarding when to destroy the SDK.
			if (appIdHasChanged && InneractiveAdManager.wasInitialized()) {
				InneractiveAdManager.destroy();
			}

			InneractiveAdManager.initialize(this, appId);
			mCurrentAppId = appId;

			/*
			 * TODO
			 * EU's General Data Protection Regulation (GDPR) consent - usage example
			 * TODO
			 *
			 *
			 * InneractiveAdManager.setGdprConsent(wasConsentGiven);
			 *
			 */

		}

	}

	/** Make sure you clear Inneractive assets when they are no longer needed */
    @Override
    protected void onDestroy() {
    	if (mToast != null) {
    		mToast.cancel();
    		mToast = null;
    	}
    	// Call inneractive::destroy only if this activity is actually finishing, and not during activity restart due to rotation etc.
        if (isFinishing()) {
			destroyDynamicAds();

			// Destroy the entire library
        	InneractiveAdManager.destroy();
        }

        super.onDestroy();
    }

	/**
	 * Called when the You choose to show a banner within the dynamic tab
	 * @param view the clicked radio button
	 */
	public void onLoadBannerClicked(View view) {
		initInneractiveSdk(bannerAppIdText != null ? bannerAppIdText.getText().toString() : SAMPLE_APP_ID);
		String spotId = SAMPLE_BANNER_SPOT_ID;
		// Get spot id from input field
		if (bannerSpotIdText != null) {
			spotId = bannerSpotIdText.getText().toString();
			if (TextUtils.isEmpty(spotId)) {
				eventToast("You didn't set your specific spot id! The sample spot id will be used. make sure that you're using your specific spot id for tracking your traffic!", Toast.LENGTH_LONG);
				spotId = SAMPLE_BANNER_SPOT_ID;
			}
			Log.d(INNERACTIVE_LOG_TAG, "Request Banner with spotId = " + spotId);
		}

		// initialize banner spot
		if (mBannerSpot != null) {
			mBannerSpot.destroy();
		}

		mBannerSpot = InneractiveAdSpotManager.get().createSpot();
		InneractiveAdViewUnitController controller = new InneractiveAdViewUnitController();
		mBannerSpot.addUnitController(controller);

		InneractiveAdRequest request = new InneractiveAdRequest(spotId);
		initializRequestOptionalParameters(mBannerSpot, request);

		mBannerSpot.setRequestListener(new InneractiveAdSpot.RequestListener() {
			@Override
			public void onInneractiveSuccessfulAdRequest(InneractiveAdSpot adSpot) {
				if (adSpot != mBannerSpot) {
					Log.d(INNERACTIVE_LOG_TAG, "Wrong Banner Spot: Received - " + adSpot + ", Actual - " + mBannerSpot);
					return;
				}

				ViewGroup layout = (ViewGroup)findViewById(R.id.ad_layout);
				InneractiveAdViewUnitController controller = (InneractiveAdViewUnitController)mBannerSpot.getSelectedUnitController();
				controller.setEventsListener(new InneractiveAdViewEventsListenerWithImpressionData() {
					@Override
					public void onAdImpression(InneractiveAdSpot adSpot, ImpressionData impressionData) {
						eventToast("onAdImpressionWithImpressionData", Toast.LENGTH_LONG);
						Log.v("ImpressionData", impressionData.toString());
					}

					@Override
					public void onAdImpression(InneractiveAdSpot adSpot) {
						onAdAvailable(adSpot, false);
						eventToast("onAdImpression", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdClicked(InneractiveAdSpot adSpot) {
						eventToast("onAdClicked", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdWillCloseInternalBrowser(InneractiveAdSpot adSpot) {
						eventToast("onAdWillCloseInternalBrowser", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdWillOpenExternalApp(InneractiveAdSpot adSpot) {
						eventToast("onAdWillOpenExternalApp", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdEnteredErrorState(InneractiveAdSpot adSpot, AdDisplayError error) {
						eventToast("onAdEnteredErrorState - " + error.getMessage(), Toast.LENGTH_LONG);
					}

					@Override
					public void onAdExpanded(InneractiveAdSpot adSpot) {
						eventToast("onAdExpanded", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdResized(InneractiveAdSpot adSpot) {
						eventToast("onAdResized", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdCollapsed(InneractiveAdSpot adSpot) {
						eventToast("onAdCollapsed", Toast.LENGTH_LONG);
					}
				});

				controller.bindView(layout);
			}

			@Override
			public void onInneractiveFailedAdRequest(InneractiveAdSpot adSpot, InneractiveErrorCode errorCode) {
				eventToast("Failed loading banner! with error: " + errorCode, Toast.LENGTH_LONG);
			}
		});

		mBannerSpot.requestAd(request);
		hideKeyboard(view);
	}

	/**
	 * Called when the You choose to show a banner within the dynamic tab
	 * @param view the clicked radio button
	 */
	public void onLoadRectangleClicked(View view) {
		initInneractiveSdk(bannerAppIdText != null ? bannerAppIdText.getText().toString() : SAMPLE_APP_ID);

		String spotId = SAMPLE_RECTANGLE_SPOT_ID;
		// Get spot id from input field
		if (bannerSpotIdText != null) {
			spotId = bannerSpotIdText.getText().toString();
			if (TextUtils.isEmpty(spotId)) {
				eventToast("You didn't set your specific spot id! The sample spot id will be used. make sure that you're using your specific spot id for tracking your traffic!", Toast.LENGTH_LONG);
				spotId = SAMPLE_RECTANGLE_SPOT_ID;
			}
			Log.d(INNERACTIVE_LOG_TAG, "Request Rectangle with spotId = " + spotId);
		}


		// destroy previous ads


		// initialize rectangle spot
		if (mBannerSpot != null) {
			mBannerSpot.destroy();
		}

		mBannerSpot = InneractiveAdSpotManager.get().createSpot();
		InneractiveAdViewUnitController controller = new InneractiveAdViewUnitController();
		mBannerSpot.addUnitController(controller);

		// If you want to get back progress and complete events, or manager the play and pause by yourself
		// you can add a video content controller, for controlling video ads
        InneractiveAdViewVideoContentController videoContentController = new InneractiveAdViewVideoContentController();
		videoContentController.setEventsListener(new VideoContentListener() {
			@Override
			public void onProgress(int totalDurationInMsec, int positionInMsec) {
				Log.d(INNERACTIVE_LOG_TAG, "Mrect: Got video content progress: total time = " + totalDurationInMsec + " position = " + positionInMsec);
			}

			@Override
			public void onCompleted() {
				Log.d(INNERACTIVE_LOG_TAG, "Mrect: Got video content completed event");
			}

			@Override
			public void onPlayerError() {
				Log.d(INNERACTIVE_LOG_TAG, "Mrect: Got video content player error event");
			}
		});

		controller.addContentController(videoContentController);

		InneractiveAdRequest request = new InneractiveAdRequest(spotId);
		initializRequestOptionalParameters(mBannerSpot, request);

		mBannerSpot.setRequestListener(new InneractiveAdSpot.RequestListener() {
			@Override
			public void onInneractiveSuccessfulAdRequest(InneractiveAdSpot adSpot) {
				if (adSpot != mBannerSpot) {
					Log.d(INNERACTIVE_LOG_TAG, "Wrong Banner Spot: Received - " + adSpot + ", Actual - " + mBannerSpot);
					return;
				}

				ViewGroup layout = (ViewGroup)findViewById(R.id.ad_layout);
				InneractiveAdViewUnitController controller = (InneractiveAdViewUnitController)mBannerSpot.getSelectedUnitController();
				controller.setEventsListener(new InneractiveAdViewEventsListener() {
					@Override
					public void onAdImpression(InneractiveAdSpot adSpot) {
						onAdAvailable(adSpot, false);
						eventToast("onAdImpression", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdClicked(InneractiveAdSpot adSpot) {
						eventToast("onAdClicked", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdWillCloseInternalBrowser(InneractiveAdSpot adSpot) {
						eventToast("onAdWillCloseInternalBrowser", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdWillOpenExternalApp(InneractiveAdSpot adSpot) {
						eventToast("onAdWillOpenExternalApp", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdEnteredErrorState(InneractiveAdSpot adSpot, AdDisplayError error) {
						eventToast("onAdEnteredErrorState - " + error.getMessage(), Toast.LENGTH_LONG);
					}

					@Override
					public void onAdExpanded(InneractiveAdSpot adSpot) {
						eventToast("onAdExpanded", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdResized(InneractiveAdSpot adSpot) {
						eventToast("onAdResized", Toast.LENGTH_LONG);
					}

					@Override
					public void onAdCollapsed(InneractiveAdSpot adSpot) {
						eventToast("onAdCollapsed", Toast.LENGTH_LONG);
					}
				});

				controller.bindView(layout);
			}

			@Override
			public void onInneractiveFailedAdRequest(InneractiveAdSpot adSpot, InneractiveErrorCode errorCode) {
				eventToast("Failed loading banner! with error: " + errorCode, Toast.LENGTH_LONG);
			}
		});

		mBannerSpot.requestAd(request);
		hideKeyboard(view);
	}

	/**
	 * Called when the LoadInterstitial button is clicked
	 * @param view button
	 */
    public void onLoadInterstitialAdClicked(View view){
		initInneractiveSdk(interstitialAppIdText != null ? interstitialAppIdText.getText().toString() : SAMPLE_APP_ID);

		String spotId = SAMPLE_INTERSTITIAL_SPOT_ID;
		// Get spot id from input field
		if (interstitialSpotIdText != null) {
			spotId = interstitialSpotIdText.getText().toString();
			if (TextUtils.isEmpty(spotId)) {
				eventToast("You didn't set your specific spot id! The sample spot id will be used. make sure that you're using your specific spot id for tracking your traffic!", Toast.LENGTH_LONG);
				spotId = SAMPLE_INTERSTITIAL_SPOT_ID;
			}
			Log.d(INNERACTIVE_LOG_TAG, "Request Interstitial with spotId = " + spotId);
		}

		///////////////////////////////////////////////////////
		// Create interstitial spot
		if (mInterstitialSpot != null) {
			mInterstitialSpot.destroy();
			mInterstitialSpot = null;
		}

		// First create a spot
		mInterstitialSpot = InneractiveAdSpotManager.get().createSpot();

		// Now create a full screen unit controller
		InneractiveFullscreenUnitController fullscreenUnitController = new InneractiveFullscreenUnitController();

		// Configure the request
		InneractiveAdRequest request = new InneractiveAdRequest(spotId);
		initializRequestOptionalParameters(mInterstitialSpot, request);

		// Add the unit controller to the spot
		mInterstitialSpot.addUnitController(fullscreenUnitController);

		mInterstitialSpot.setRequestListener(new InneractiveAdSpot.RequestListener() {
			@Override
			public void onInneractiveSuccessfulAdRequest(InneractiveAdSpot adSpot) {
				eventToast("Interstitial loaded successfully!", Toast.LENGTH_LONG);
				onAdAvailable(adSpot,true);
			}

			@Override
			public void onInneractiveFailedAdRequest(InneractiveAdSpot adSpot, InneractiveErrorCode errorCode) {
				eventToast("Failed loading interstitial! with error: " + errorCode, Toast.LENGTH_LONG);
			}
		});

		mInterstitialSpot.requestAd(request);
		hideKeyboard(view);
    }

	/**
	 * Called when the show interstitial button is clicked
	 * @param view button
	 */
    public void onShowInterstitialAdClicked(View view){
		if (mInterstitialSpot != null && mInterstitialSpot.isReady()) {
			InneractiveFullscreenUnitController fullscreenUnitController = (InneractiveFullscreenUnitController)mInterstitialSpot.getSelectedUnitController();
			fullscreenUnitController.setEventsListener(new InneractiveFullscreenAdEventsListenerWithImpressionData() {
				@Override
				public void onAdImpression(InneractiveAdSpot adSpot, ImpressionData impressionData) {
					eventToast("onAdImpressionWithImpressionData", Toast.LENGTH_LONG);
					Log.v("ImpressionData", impressionData.toString());
				}

				@Override
				public void onAdImpression(InneractiveAdSpot adSpot) {
					onAdAvailable(adSpot, false);
					eventToast("onAdImpression", Toast.LENGTH_LONG);
				}

				@Override
				public void onAdDismissed(InneractiveAdSpot adSpot) {
					eventToast("onAdDismissed", Toast.LENGTH_LONG);
				}


				@Override
				public void onAdClicked(InneractiveAdSpot adSpot) {
					eventToast("onAdClicked", Toast.LENGTH_LONG);
				}

				@Override
				public void onAdWillOpenExternalApp(InneractiveAdSpot adSpot) {
					eventToast("onAdWillOpenExternalApp", Toast.LENGTH_LONG);
				}

				@Override
				public void onAdEnteredErrorState(InneractiveAdSpot adSpot, AdDisplayError error) {
					eventToast("onAdEnteredErrorState - " + error.getMessage(), Toast.LENGTH_LONG);
				}

				@Override
				public void onAdWillCloseInternalBrowser(InneractiveAdSpot adSpot) {
					eventToast("onAdWillCloseInternalBrowser", Toast.LENGTH_LONG);
				}
			});

			// Add video content controller, for controlling video ads
			InneractiveFullscreenVideoContentController videoContentController = new InneractiveFullscreenVideoContentController();
			videoContentController.setEventsListener(new VideoContentListener() {
				@Override
				public void onProgress(int totalDurationInMsec, int positionInMsec) {
					Log.d(INNERACTIVE_LOG_TAG, "Interstitial: Got video content progress: total time = " + totalDurationInMsec + " position = " + positionInMsec);
				}

				@Override
				public void onCompleted() {
					Log.d(INNERACTIVE_LOG_TAG, "Interstitial: Got video content completed event");
				}

				@Override
				public void onPlayerError() {
					Log.d(INNERACTIVE_LOG_TAG, "Interstitial: Got video content player error event");
				}
			});

			// If you would like to change the full screen video behaviour, you can create and configure a video content controller, and add it to the unit controller
			// Override the default behaviour - show the controls within the video frame
			videoContentController.setOverlayOutside(false);

			// Now add the content controller to the unit controller
			fullscreenUnitController.addContentController(videoContentController);

			fullscreenUnitController.show(this);
		} else {
			eventToast("The Interstitial ad is not ready yet.", Toast.LENGTH_SHORT);
		}
    }

	///////////////////////////////////
	// Internal helper methods

	/**
	 * Example how to initialize the Ad with request parameters
	 * <br>Improves the ad targeting
	 * @param adRequest ad request, which will be used for requestAd
	 */
    protected void initializRequestOptionalParameters(InneractiveAdSpot adSpot, InneractiveAdRequest adRequest) {
		// Add user related data
    	adRequest.setUserParams( new InneractiveUserConfig()
		.setGender(InneractiveUserConfig.Gender.FEMALE)
		.setZipCode("94401")
		.setAge(35));

		// Add keywords. Separated by a comma
		adRequest.setKeywords("pop,rock,music");
	}

	/**
	 * Helper for displaying a toast message.
	 * <br>The Toast is cached, so we can close it immediately if we get out
	 * @param message
	 * @param length
	 */
    protected void eventToast(String message, int length) {
		mToast = Toast.makeText(this, message, length);
		mToast.show();

		// Also log
		Log.d(INNERACTIVE_LOG_TAG, message);
    }

	/** Destroy 1st tab ads */
	private void destroyDynamicAds() {
		if (mBannerSpot != null) {
			mBannerSpot.destroy();
			mBannerSpot = null;
		}

		if (mInterstitialSpot != null) {
			mInterstitialSpot.destroy();
			mInterstitialSpot = null;
		}
	}

	protected void onAdAvailable(InneractiveAdSpot adSpot, boolean isAdLoaded) {
	}

	/**
	 * Small helper for updating the Sample title
	 * @param title
	 */
	protected void updateTitle(String title) {
		TextView header = (TextView) findViewById(R.id.header_text);
		if (header != null) {
			header.setText(title);
		}
	}

	protected void hideKeyboard(View view) {
		InputMethodManager imm = (InputMethodManager)getSystemService(
				Context.INPUT_METHOD_SERVICE);
		imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
	}
}
